#include "stdafx.h"
#include <string.h>
#include "ECRReader.h"
#include "BaseCommunication.h"

#define base CECPReader

CECRReader::CECRReader(CReader *Owner,CBaseCommunication *Communicator)
			  :base(Owner,Communicator)
{
}

CECRReader::~CECRReader(void)
{
}

void CECRReader::SetHWString(char *String)
{
   strcpy(String,"ECR_");
}

void CECRReader::GetProductString(uint8_t *Product)
{
	memcpy(Product,"CJECR",5);
}

RSCT_IFD_RESULT CECRReader::IfdPower(uint32_t Mode, uint8_t *ATR, uint32_t *ATR_Length, uint32_t Timeout)
{
	uint32_t OrginalATR_Length=0;
	if(ATR_Length!=0)
		OrginalATR_Length=*ATR_Length;
	RSCT_IFD_RESULT Result=base::IfdPower(Mode,ATR,ATR_Length,Timeout);
	m_bIsRF=false;
	if(Result==STATUS_SUCCESS && Mode==SCARD_COLD_RESET || Mode==SCARD_WARM_RESET)
	{
		if(m_ATR_Length>=6 && m_ATR_Length<=12)
		{
			if(m_ATR[0]==0x41)
			{
				bool bDoSelectTCL=true;
				uint8_t tck=0;

				m_SerialNumberLength=m_ATR_Length-3;
				memcpy(m_SerialNumber,m_ATR+3,m_SerialNumberLength);
				if(m_ATR[1] & 0x20)
				{
					if(m_ATR[1]==0x28 || m_ATR[1]==0x38)
					{
						if(GetEnviroment("PrefereMifareClasic",0))
						{
							bDoSelectTCL=false;
						}
					}
					if(bDoSelectTCL)
					{
						CCID_Message Message;
						CCID_Response Response;

						memset(&Message,0,sizeof(Message));
						Message.bMessageType=PC_TO_RDR_SETPARAMETERS;
          			Message.Header.SetParameters.bProtocolNum=3;//Get ATS
						switch(Transfer(&Message,&Response))
						{
						case CJ_SUCCESS:
							if((Response.bStatus&3)==2)
								return STATUS_NO_MEDIA;
							else if((Response.bStatus&3)==1 || (Response.bStatus & 0x40))
								return STATUS_IO_TIMEOUT;
							memcpy(m_ATR,"\x3b\x80\x80\x01",4);
							if(Response.Data.abData[0]>15)
								return STATUS_NO_MEDIA;
							m_ATR[1]|=Response.Data.abData[0];
							memcpy(m_ATR+4,Response.Data.abData+1,Response.Data.abData[0]);
							m_ATR_Length=Response.Data.abData[0]+4;

							break;
						default:
							return STATUS_DEVICE_NOT_CONNECTED;
						}
					}
				}
				else
					bDoSelectTCL=false;
				if(!bDoSelectTCL)
				{
					uint8_t merkSAK=m_ATR[1];
				   memcpy(m_ATR,"\x3b\x8f\x80\x01\x80\x4f\x0c\xA0\x00\x00\x03\x06\x00\x00\x00\x00\x00\x00\x00",19);
					m_ATR_Length=19;
					switch(merkSAK & 0xdf)
					{
					case 0x08:
						memcpy(m_ATR+12,"\x03\x00\x01",3);
						break;
					case 0x09:
						memcpy(m_ATR+12,"\x03\x00\x26",3);
						break;
					case 0x18:
						memcpy(m_ATR+12,"\x03\x00\x02",3);
						break;
					default:;
					}
				}
				for(unsigned int i=1;i<m_ATR_Length;i++)
				{
					tck^=m_ATR[i];
				}
				m_ATR[m_ATR_Length]=tck;
				m_ATR_Length++;
				if(OrginalATR_Length<m_ATR_Length)
					return STATUS_BUFFER_TOO_SMALL;
				memcpy(ATR,m_ATR,m_ATR_Length);
				*ATR_Length=m_ATR_Length;
				m_PossibleProtocols=SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1;
				m_ActiveProtocol=0;
   		   m_ReaderState=SCARD_NEGOTIABLE;
				m_bIsRF=true;
			}
		}
	}

	return Result;

}

RSCT_IFD_RESULT CECRReader::IfdSetProtocol(uint32_t *Protocol)
{
	if(m_bIsRF)
	{
		m_ActiveProtocol=*Protocol;
		m_ReaderState=SCARD_SPECIFIC;
		return STATUS_SUCCESS;
	}
	return base::IfdSetProtocol(Protocol);
}

RSCT_IFD_RESULT CECRReader::_IfdTransmit(const uint8_t *cmd, uint16_t cmd_len,uint8_t *response,uint16_t *response_len)
{
	if(m_bIsRF && cmd_len==5 && cmd[0]==0xff && cmd[1]==0xca && cmd[2]<=0x01 && cmd[3]==0)
	{
		if(cmd[2]==0)
		{
			if(*response_len>=m_SerialNumberLength+2)
			{
				if(cmd[4]==0 || cmd[4]>=m_SerialNumberLength)
				{
				   memcpy(response,m_SerialNumber,m_SerialNumberLength);
					if(cmd[4]!=0 && cmd[4]>m_SerialNumberLength)
					{
				      memset(response+m_SerialNumberLength,0,cmd[4]-m_SerialNumberLength);
				      memcpy(response+cmd[4],"\x62\x82",2);
					   *response_len=cmd[4]+2;
					}
					else
					{
				      memcpy(response+m_SerialNumberLength,"\x90\x00",2);
					   *response_len=m_SerialNumberLength+2;
					}
					return STATUS_SUCCESS;
				}
			}
			if(*response_len>=2)
			{
			   response[0]=0x6C;
			   response[1]=m_SerialNumberLength;
				*response_len=2;
				return STATUS_SUCCESS;
			}
   		return STATUS_BUFFER_TOO_SMALL;
		}
		else
		{
			if(*response_len>=m_ATR_Length-3)
			{
				if(cmd[4]==0 || cmd[4]>=m_ATR_Length-5)
				{
				   memcpy(response,m_ATR+4,m_ATR_Length-5);
					if(cmd[4]!=0 && cmd[4]>m_ATR_Length-5)
					{
				      memset(response+m_ATR_Length-5,0,cmd[4]-m_ATR_Length+5);
				      memcpy(response+cmd[4],"\x62\x82",2);
					   *response_len=cmd[4]+2;
					}
					else
					{
				      memcpy(response+m_ATR_Length-5,"\x90\x00",2);
					   *response_len=(uint16_t)(m_ATR_Length-3);
					}
					return STATUS_SUCCESS;
				}
			}
			if(*response_len>=2)
			{
			   response[0]=0x6C;
			   response[1]=(uint8_t)(m_ATR_Length-5);
				*response_len=2;
				return STATUS_SUCCESS;
			}
   		return STATUS_BUFFER_TOO_SMALL;
		}
	}
	return base::_IfdTransmit(cmd,cmd_len,response,response_len);
}



